@@ -1,7 +1,7 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 # Copyright 2013 IBM Corporation
-# Copyright 2015 Lenovo
+# Copyright 2015-2017 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,12 @@ import pyghmi.exceptions as exc
 import pyghmi.ipmi.events as sel
 import pyghmi.ipmi.fru as fru
 from pyghmi.ipmi.oem.lookup import get_oem_handler
-from pyghmi.ipmi.private import session
+
+try:
+    from pyghmi.ipmi.private import session
+except ImportError:
+    session = None
+from pyghmi.ipmi.private import localsession
 import pyghmi.ipmi.private.util as pygutil
 import pyghmi.ipmi.sdr as sdr
 import socket
@@ -38,7 +43,6 @@ try:
 except NameError:
     buffer = memoryview
 
-
 boot_devices = {
     'net': 4,
     'network': 4,
@@ -86,7 +90,7 @@ def _mask_to_cidr(mask):
 
 
 def _cidr_to_mask(prefix):
-    return struct.pack('>I', 2**prefix-1 << (32-prefix))
+    return struct.pack('>I', 2 ** prefix - 1 << (32 - prefix))
 
 
 class Command(object):
@@ -103,15 +107,16 @@ class Command(object):
     callback_args parameter. However, callback_args can optionally be populated
     if desired.
 
-    :param bmc: hostname or ip address of the BMC
-    :param userid: username to use to connect
-    :param password: password to connect to the BMC
+    :param bmc: hostname or ip address of the BMC (default is local)
+    :param userid: username to use to connect (default to no user)
+    :param password: password to connect to the BMC (defaults to no password)
     :param onlogon: function to run when logon completes in an asynchronous
                     fashion.  This will result in a greenthread behavior.
     :param kg: Optional parameter to use if BMC has a particular Kg configured
     """
 
-    def __init__(self, bmc, userid, password, port=623, onlogon=None, kg=None):
+    def __init__(self, bmc=None, userid=None, password=None, port=623,
+                 onlogon=None, kg=None):
         # TODO(jbjohnso): accept tuples and lists of each parameter for mass
         # operations without pushing the async complexities up the stack
         self.onlogon = onlogon
@@ -121,7 +126,9 @@ class Command(object):
         self._netchannel = None
         self._ipv6support = None
         self.certverify = None
-        if onlogon is not None:
+        if bmc is None:
+            self.ipmi_session = localsession.Session()
+        elif onlogon is not None:
             self.ipmi_session = session.Session(bmc=bmc,
                                                 userid=userid,
                                                 password=password,
@@ -428,7 +435,7 @@ class Command(object):
         response = self.raw_command(netfn=0, command=1)
         if 'error' in response:
             raise exc.IpmiException(response['error'])
-        assert(response['command'] == 1 and response['netfn'] == 1)
+        assert (response['command'] == 1 and response['netfn'] == 1)
         powerstate = 'on' if (response['data'][0] & 1) else 'off'
         return {'powerstate': powerstate}
 
@@ -756,7 +763,7 @@ class Command(object):
             1: 'Static',
             2: 'DHCP',
             3: 'BIOS',
-            4:  'Other',
+            4: 'Other',
         }
         retdata['ipv4_configuration'] = v4cfgmethods[self._fetch_lancfg_param(
             channel, 4)]
@@ -818,7 +825,7 @@ class Command(object):
         that
         """
         if self._netchannel is None:
-            for channel in chain((0xe, ), range(1, 0xc)):
+            for channel in chain((0xe,), range(1, 0xc)):
                 try:
                     rsp = self.xraw_command(
                         netfn=6, command=0x42, data=(channel,))
@@ -834,8 +841,11 @@ class Command(object):
                     try:
                         # Some implementations denote an inactive channel
                         # by refusing to do parameter retrieval
-                        self.xraw_command(
-                            netfn=0xc, command=2, data=(channel, 5, 0, 0))
+                        if channel != 0xe:
+                            # skip checking if channel is active if we are
+                            # actively using the channel
+                            self.xraw_command(
+                                netfn=0xc, command=2, data=(channel, 5, 0, 0))
                         # If still here, the channel seems serviceable...
                         # However some implementations may still have
                         # ambiguous channel info, that will need to be
@@ -1095,7 +1105,7 @@ class Command(object):
         return retstr
 
     def _chunkwise_dcmi_set(self, command, data):
-        chunks = [data[i:i+15] for i in range(0, len(data), 15)]
+        chunks = [data[i:i + 15] for i in range(0, len(data), 15)]
         offset = 0
         for chunk in chunks:
             chunk = bytearray(chunk, 'utf-8')
@@ -1174,7 +1184,7 @@ class Command(object):
             'dont_change': 0,
             'non_volatile': 1,
             'volatile': 2,
-            #'reserved': 3
+            # 'reserved': 3
         }
         b = 0
         b |= (access_update_modes[access_update_mode] << 6) & 0b11000000
@@ -1197,7 +1207,7 @@ class Command(object):
             'dont_change': 0,
             'non_volatile': 1,
             'volatile': 2,
-            #'reserved': 3
+            # 'reserved': 3
         }
         b |= (privilege_update_modes[privilege_update_mode] << 6) & 0b11000000
         privilege_levels = {
@@ -1282,7 +1292,7 @@ class Command(object):
             3: 'operator',
             4: 'administrator',
             5: 'proprietary',
-            #0x0F: 'no_access'
+            # 0x0F: 'no_access'
         }
         r['privilege_level'] = privilege_levels[data[1] & 0b00001111]
         return r
@@ -1327,8 +1337,8 @@ class Command(object):
             0x0a: 'reserved for USB 1.x',
             0x0b: 'reserved for USB 2.x',
             0x0c: 'System Interface (KCS, SMIC, or BT)',
-            ## 60h-7Fh: OEM
-            ## all other  reserved
+            # 60h-7Fh: OEM
+            # all other  reserved
         }
         t = data[1] & 0b01111111
         if t in channel_medium_types:
@@ -1440,7 +1450,7 @@ class Command(object):
             privilege_level: [reserved, callback, user,
                               operatorm administrator, proprietary, no_access]
         """
-        ## user access available during call-in or callback direct connection
+        # user access available during call-in or callback direct connection
         if channel is None:
             channel = self.get_network_channel()
         data = [channel, uid]
@@ -1480,9 +1490,7 @@ class Command(object):
             raise Exception('name must be less than or = 16 chars')
         name = name.ljust(16, "\x00")
         data.extend([ord(x) for x in name])
-        response = self.raw_command(netfn=0x06, command=0x45, data=data)
-        if 'error' in response:
-            raise Exception(response['error'])
+        self.xraw_command(netfn=0x06, command=0x45, data=data)
         return True
 
     def get_user_name(self, uid, return_none_on_error=True):
@@ -1544,12 +1552,16 @@ class Command(object):
             else:
                 password = password.ljust(16, "\x00")
             data.extend([ord(x) for x in password])
-        response = self.raw_command(netfn=0x06, command=0x47, data=data)
-        if 'error' in response:
+        try:
+            self.xraw_command(netfn=0x06, command=0x47, data=data)
+        except exc.IpmiException as ie:
             if mode == 'test_password':
-                # return false if password test failed
                 return False
-            raise Exception(response['error'])
+            elif mode in ('enable', 'disable') and ie.ipmicode == 0xcc:
+                # Some BMCs see redundant calls to password disable/enable
+                # as invalid
+                return True
+            raise
         return True
 
     def get_channel_max_user_count(self, channel=None):
@@ -1624,7 +1636,7 @@ class Command(object):
             channel = self.get_network_channel()
         names = {}
         max_ids = self.get_channel_max_user_count(channel)
-        for uid in range(1, max_ids+1):
+        for uid in range(1, max_ids + 1):
             name = self.get_user_name(uid=uid)
             if name is not None:
                 names[uid] = self.get_user(uid=uid, channel=channel)
@@ -1679,7 +1691,9 @@ class Command(object):
         try:
             # First try to set name to all \x00 explicitly
             self.set_user_name(uid, '')
-        except Exception:
+        except exc.IpmiException as ie:
+            if ie.ipmicode != 0xcc:
+                raise
             # An invalid data field in request  is frequently reported.
             # however another convention that exists is all '\xff'
             # if this fails, pass up the error so that calling code knows
