@@ -18,9 +18,14 @@
 
 import pyghmi.exceptions as exc
 import struct
+import threading
 
 from pyghmi.ipmi.private import constants
 from pyghmi.ipmi.private import session
+from pyghmi.ipmi.private import util
+
+
+PENDINGOUTPUT = threading.RLock()
 
 
 class Console(object):
@@ -38,7 +43,7 @@ class Console(object):
     :param kg: optional parameter for BMCs configured to require it
     """
 
-    #TODO(jbjohnso): still need an exit and a data callin function
+    # TODO(jbjohnso): still need an exit and a data callin function
     def __init__(self, bmc, userid, password,
                  iohandler, port=623,
                  force=False, kg=None):
@@ -70,19 +75,19 @@ class Console(object):
         if 'error' in response:
             self._print_error(response['error'])
             return
-        #Send activate sol payload directive
-        #netfn= 6 (application)
-        #command = 0x48 (activate payload)
-        #data = (1, sol payload type
+        # Send activate sol payload directive
+        # netfn= 6 (application)
+        # command = 0x48 (activate payload)
+        # data = (1, sol payload type
         #        1, first instance
         #        0b11000000, -encrypt, authenticate,
         #                      disable serial/modem alerts, CTS fine
         #        0, 0, 0 reserved
         response = self.ipmi_session.raw_command(netfn=0x6, command=0x48,
                                                  data=(1, 1, 192, 0, 0, 0))
-        #given that these are specific to the command,
-        #it's probably best if one can grep the error
-        #here instead of in constants
+        # given that these are specific to the command,
+        # it's probably best if one can grep the error
+        # here instead of in constants
         sol_activate_codes = {
             0x81: 'SOL is disabled',
             0x82: 'Maximum SOL session count reached',
@@ -118,19 +123,19 @@ class Console(object):
             self._print_error(response['error'])
             return
         self.activated = True
-        #data[0:3] is reserved except for the test mode, which we don't use
+        # data[0:3] is reserved except for the test mode, which we don't use
         data = response['data']
         self.maxoutcount = (data[5] << 8) + data[4]
-           #BMC tells us this is the maximum allowed size
-        #data[6:7] is the promise of how small packets are going to be, but we
-        #don't have any reason to worry about it
+        # BMC tells us this is the maximum allowed size
+        # data[6:7] is the promise of how small packets are going to be, but we
+        # don't have any reason to worry about it
         if (data[8] + (data[9] << 8)) not in (623, 28418):
-            #TODO(jbjohnso): support atypical SOL port number
+            # TODO(jbjohnso): support atypical SOL port number
             raise NotImplementedError("Non-standard SOL Port Number")
-        #ignore data[10:11] for now, the vlan detail, shouldn't matter to this
-        #code anyway...
-        #NOTE(jbjohnso):
-        #We will use a special purpose keepalive
+        # ignore data[10:11] for now, the vlan detail, shouldn't matter to this
+        # code anyway...
+        # NOTE(jbjohnso):
+        # We will use a special purpose keepalive
         if self.ipmi_session.sol_handler is not None:
             # If there is erroneously another SOL handler already, notify
             # it of newly established session
@@ -140,8 +145,8 @@ class Console(object):
             callback=self._got_payload_instance_info)
         self.ipmi_session.sol_handler = self._got_sol_payload
         self.connected = True
-        if len(self.pendingoutput) > 0:
-            self._sendpendingoutput()
+        # self._sendpendingoutput() checks len(self._sendpendingoutput)
+        self._sendpendingoutput()
 
     def _got_payload_instance_info(self, response):
         if 'error' in response:
@@ -162,6 +167,7 @@ class Console(object):
         # discern which channel or even *if* the serial port in question
         # correlates at all to an ipmi channel to check mux
 
+    @util.protect(PENDINGOUTPUT)
     def _addpendingdata(self, data):
         if isinstance(data, dict):
             self.pendingoutput.append(data)
@@ -219,7 +225,10 @@ class Console(object):
         """
         return session.Session.wait_for_rsp(timeout=timeout)
 
+    @util.protect(PENDINGOUTPUT)
     def _sendpendingoutput(self):
+        if len(self.pendingoutput) == 0:
+            return
         if isinstance(self.pendingoutput[0], dict):
             if 'break' in self.pendingoutput[0]:
                 self._sendoutput("", sendbreak=True)
@@ -296,9 +305,9 @@ class Console(object):
     def _got_sol_payload(self, payload):
         """SOL payload callback
         """
-        #TODO(jbjohnso) test cases to throw some likely scenarios at functions
-        #for example, retry with new data, retry with no new data
-        #retry with unexpected sequence number
+        # TODO(jbjohnso) test cases to throw some likely scenarios at functions
+        # for example, retry with new data, retry with no new data
+        # retry with unexpected sequence number
         if type(payload) == dict:  # we received an error condition
             self.activated = False
             self._print_error(payload)
@@ -310,15 +319,16 @@ class Console(object):
         poweredoff = payload[3] & 0b100000
         deactivated = payload[3] & 0b10000
         breakdetected = payload[3] & 0b100
-        #for now, ignore overrun.  I assume partial NACK for this reason or for
-        #no reason would be treated the same, new payload with partial data
+        # for now, ignore overrun.  I assume partial NACK for this reason or
+        # for no reason would be treated the same, new payload with partial
+        # data.
         remdata = ""
         remdatalen = 0
         if newseq != 0:  # this packet at least has some data to send to us..
             if len(payload) > 4:
                 remdatalen = len(payload[4:])  # store remote len before dupe
-                    #retry logic, we must ack *this* many even if it is
-                    #a retry packet with new partial data
+                # retry logic, we must ack *this* many even if it is
+                # a retry packet with new partial data
                 remdata = struct.pack("%dB" % remdatalen, *payload[4:])
             if newseq == self.remseq:  # it is a retry, but could have new data
                 if remdatalen > self.lastsize:
@@ -331,16 +341,16 @@ class Console(object):
             if remdata:  # Do not subject callers to empty data
                 self._print_data(remdata)
             ackpayload = (0, self.remseq, remdatalen, 0)
-            #Why not put pending data into the ack? because it's rare
-            #and might be hard to decide what to do in the context of
-            #retry situation
+            # Why not put pending data into the ack? because it's rare
+            # and might be hard to decide what to do in the context of
+            # retry situation
             try:
                 self.send_payload(ackpayload, retry=False)
             except exc.IpmiException:
-                #if the session is broken, then close the SOL session
+                # if the session is broken, then close the SOL session
                 self.close()
         if self.myseq != 0 and ackseq == self.myseq:  # the bmc has something
-                                                      # to say about last xmit
+            # to say about last xmit
             self.awaitingack = False
             if nacked and not breakdetected:  # the BMC was in some way unhappy
                 if poweredoff:
@@ -352,14 +362,15 @@ class Console(object):
                     # also add pending output for efficiency and ease
                     newtext = self.lastpayload[4 + ackcount:]
                     newtext = struct.pack("B"*len(newtext), *newtext)
-                    if (self.pendingoutput and
-                            not isinstance(self.pendingoutput[0], dict)):
-                        self.pendingoutput[0] = newtext + self.pendingoutput[0]
-                    else:
-                        self.pendingoutput = [newtext] + self.pendingoutput
-                    self._sendpendingoutput()
-            if len(self.pendingoutput) > 0:
-                self._sendpendingoutput()
+                    with util.protect(PENDINGOUTPUT):
+                        if (self.pendingoutput and
+                                not isinstance(self.pendingoutput[0], dict)):
+                            self.pendingoutput[0] = \
+                                newtext + self.pendingoutput[0]
+                        else:
+                            self.pendingoutput = [newtext] + self.pendingoutput
+            # self._sendpendingoutput() checks len(self._sendpendingoutput)
+            self._sendpendingoutput()
         elif ackseq != 0 and self.awaitingack:
             # if an ack packet came in, but did not match what we
             # expected, retry our payload now.
@@ -379,9 +390,129 @@ class Console(object):
         to provide their own event loop behavior, though this could be used
         within the greenthread implementation of caller's choice if desired.
         """
-        #wait_for_rsp promises to return a false value when no sessions are
-        #alive anymore
-        #TODO(jbjohnso): wait_for_rsp is not returning a true value for our own
-        #session
+        # wait_for_rsp promises to return a false value when no sessions are
+        # alive anymore
+        # TODO(jbjohnso): wait_for_rsp is not returning a true value for our
+        # own session
         while (1):
             session.Session.wait_for_rsp(timeout=600)
+
+
+class ServerConsole(Console):
+    """IPMI SOL class.
+
+    This object represents an SOL channel, multiplexing SOL data with
+    commands issued by ipmi.command.
+
+    :param session: IPMI session
+    :param iohandler: I/O handler
+    """
+
+    def __init__(self, _session, iohandler, force=False):
+        self.keepaliveid = None
+        self.connected = True
+        self.broken = False
+        self.out_handler = iohandler
+        self.remseq = 0
+        self.myseq = 0
+        self.lastsize = 0
+        self.retriedpayload = 0
+        self.pendingoutput = []
+        self.awaitingack = False
+        self.activated = True
+        self.force_session = force
+        self.ipmi_session = _session
+        self.ipmi_session.sol_handler = self._got_sol_payload
+        self.maxoutcount = 256
+        self.poweredon = True
+
+        session.Session.wait_for_rsp(0)
+
+    def _got_sol_payload(self, payload):
+        """SOL payload callback
+        """
+        # TODO(jbjohnso) test cases to throw some likely scenarios at functions
+        # for example, retry with new data, retry with no new data
+        # retry with unexpected sequence number
+        if type(payload) == dict:  # we received an error condition
+            self.activated = False
+            self._print_error(payload)
+            return
+        newseq = payload[0] & 0b1111
+        ackseq = payload[1] & 0b1111
+        ackcount = payload[2]
+        nacked = payload[3] & 0b1000000
+        breakdetected = payload[3] & 0b10000
+        # for now, ignore overrun.  I assume partial NACK for this reason or
+        # for no reason would be treated the same, new payload with partial
+        # data.
+        remdata = ""
+        remdatalen = 0
+        flag = 0
+        if not self.poweredon:
+            flag |= 0b1100000
+        if not self.activated:
+            flag |= 0b1010000
+        if newseq != 0:  # this packet at least has some data to send to us..
+            if len(payload) > 4:
+                remdatalen = len(payload[4:])  # store remote len before dupe
+                # retry logic, we must ack *this* many even if it is
+                # a retry packet with new partial data
+                remdata = struct.pack("%dB" % remdatalen, *payload[4:])
+            if newseq == self.remseq:  # it is a retry, but could have new data
+                if remdatalen > self.lastsize:
+                    remdata = remdata[4 + self.lastsize:]
+                else:  # no new data...
+                    remdata = ""
+            else:  # TODO(jbjohnso) what if remote sequence number is wrong??
+                self.remseq = newseq
+            self.lastsize = remdatalen
+            ackpayload = (0, self.remseq, remdatalen, flag)
+            # Why not put pending data into the ack? because it's rare
+            # and might be hard to decide what to do in the context of
+            # retry situation
+            try:
+                self.send_payload(ackpayload, retry=False)
+            except exc.IpmiException:
+                # if the session is broken, then close the SOL session
+                self.close()
+            if remdata:  # Do not subject callers to empty data
+                self._print_data(remdata)
+        if self.myseq != 0 and ackseq == self.myseq:  # the bmc has something
+            # to say about last xmit
+            self.awaitingack = False
+            if nacked and not breakdetected:  # the BMC was in some way unhappy
+                newtext = self.lastpayload[4 + ackcount:]
+                newtext = struct.pack("B"*len(newtext), *newtext)
+                with util.protect(PENDINGOUTPUT):
+                    if (self.pendingoutput and
+                            not isinstance(self.pendingoutput[0], dict)):
+                        self.pendingoutput[0] = newtext + self.pendingoutput[0]
+                    else:
+                        self.pendingoutput = [newtext] + self.pendingoutput
+            # self._sendpendingoutput() checks len(self._sendpendingoutput)
+            self._sendpendingoutput()
+        elif ackseq != 0 and self.awaitingack:
+            # if an ack packet came in, but did not match what we
+            # expected, retry our payload now.
+            # the situation that was triggered was a senseless retry
+            # when data came in while we xmitted.  In theory, a BMC
+            # should handle a retry correctly, but some do not, so
+            # try to mitigate by avoiding overeager retries
+            # occasional retry of a packet
+            # sooner than timeout suggests is evidently a big deal
+            self.send_payload(payload=self.lastpayload)
+
+    def send_payload(self, payload, payload_type=1, retry=True,
+                     needskeepalive=False):
+        while not (self.connected or self.broken):
+            session.Session.wait_for_rsp(timeout=10)
+        self.ipmi_session.send_payload(payload,
+                                       payload_type=payload_type,
+                                       retry=retry,
+                                       needskeepalive=needskeepalive)
+
+    def close(self):
+        """Shut down an SOL session,
+        """
+        self.activated = False
